@@ -7,7 +7,7 @@ use diagnostics;
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw( auction );
-our $VERSION = '0.83';
+our $VERSION = '0.84';
 
 #Variables global to the package	
 my $maximize_total_benefit;
@@ -443,6 +443,8 @@ sub auctionRound {
 	my %this_person_can_choose_n_different_objects;
 	my %choose_object;
 	my %info;
+	my %seen_object;
+	my %count_object;
 
 	if ( $verbose >= 8 ){
 		print $output "\n Start: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; epsilon_scaling = $epsilon_scaling ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
@@ -623,22 +625,19 @@ sub auctionRound {
 			if ( $Opt01ValForPersonI == $Opt02ValForPersonI ) # this person can choose different objects that have the same values = $Opt01ValForPersonI
 			{
 				@objects_with_same_values = grep { $current_value{$_} == $Opt01ValForPersonI } keys %current_value;
-				
-				my $num_objects = scalar @objects_with_same_values;
-				#$this_person_can_choose_n_different_objects{$num_objects}{$person}{'bid'   } = $epsilon;
-				$this_person_can_choose_n_different_objects{$num_objects}{$person}{'CurVal' } = $Opt01ValForPersonI;
-				$this_person_can_choose_n_different_objects{$num_objects}{$person}{'objects'} = \@objects_with_same_values; #reference to an array
+				$seen_object{$_}{$person} = $Opt01ValForPersonI for (@objects_with_same_values);
+				$count_object{$_}++ for (@objects_with_same_values);
 				
 				if ( not @updated_price and defined $Opt01ValForPersonI_old_list ){
 					for my $object ( @objects_with_same_values ){
-						next unless ( $current_value{$object} > $Opt01ValForPersonI_old_list );
-						$objects_desired_by_this{$person}{$object} = $current_value{$object};
+						next unless ( $current_value{$object} > $Opt01ValForPersonI_old_list ); # objects in the new list that have values greater than the objects in the old list
+						$objects_desired_by_this{$person}{$object} = $current_value{$object};   # add information about the most desired objects
 					}
 				}
 
 			} else 
 			{
-				$choose_object{$Opt01ObjForPersonI}++;						
+				$choose_object{$Opt01ObjForPersonI}++;
 			
 				if ( (not defined $info{$Opt01ObjForPersonI}) || ($bidForPersonI > $info{$Opt01ObjForPersonI}{'bid'}) || ($bidForPersonI == $info{$Opt01ObjForPersonI}{'bid'} and $current_value{$Opt01ObjForPersonI} > $info{$Opt01ObjForPersonI}{'CurVal'}) )
 				{			
@@ -667,7 +666,7 @@ sub auctionRound {
 					$objects_desired_by_this{$person}{$object} = $current_value{$object}; # update current values for all objects in the old list	
 				}
 				
-				$objects_desired_by_this{$person}{$Opt01ObjForPersonI} = $current_value{$Opt01ObjForPersonI}; # information about the most desired objects
+				$objects_desired_by_this{$person}{$Opt01ObjForPersonI} = $current_value{$Opt01ObjForPersonI}; # add information about the most desired objects
 				$objects_desired_by_this{$person}{$Opt02ObjForPersonI} = $current_value{$Opt02ObjForPersonI} if (defined $Opt02ObjForPersonI);
 				$objects_desired_by_this{$person}{$Opt03ObjForPersonI} = $current_value{$Opt03ObjForPersonI} if (defined $Opt03ObjForPersonI);				
 
@@ -680,9 +679,11 @@ sub auctionRound {
 				my @old_list = sort { $a <=> $b } @objects_with_greater_benefits;
 				my @new_list = sort { $a <=> $b } keys %{$objects_desired_by_this{$person}};
 				@updated_price = sort { $a <=> $b } @updated_price;
-				my @best_3_objects = ( $Opt01ObjForPersonI, $Opt02ObjForPersonI, $Opt03ObjForPersonI );
-				@objects_with_same_values = sort { $a <=> $b } @objects_with_same_values;
+				my @best_3_objects = ( $Opt01ObjForPersonI, $Opt02ObjForPersonI, $Opt03ObjForPersonI );				
 				my $msg = $locked_list{$person} ? '[locked list] ' : '';
+				
+				@objects_with_same_values = sort { $a <=> $b } @objects_with_same_values;
+				$this_person_can_choose_n_different_objects{$person}{'objects'} = \@objects_with_same_values; #reference to an array								
 
 				printf $output "<> PersonI = %3s ; %3s objects desired by this person (old list) = (@old_list) ; objects whose current values are still updated = (@updated_price) : %2s >= 1 ? \n", $person, scalar @old_list, scalar @updated_price;
 				printf $output "<> PersonI = %3s ; %3s objects desired by this person (new list) = (@new_list) $msg; \@best_3_objects = (@best_3_objects) \n", $person, scalar @new_list if ( defined $Opt03ObjForPersonI );
@@ -691,34 +692,37 @@ sub auctionRound {
 			}
 		}
 	}
-				
-    foreach my $num_objects ( sort { $a <=> $b } keys %this_person_can_choose_n_different_objects ){ # $num_objects >= 2
-	foreach my $person      ( sort { $price_object{$b} <=> $price_object{$a} } keys %{$this_person_can_choose_n_different_objects{$num_objects}} ){
-		my $Opt01ObjForPersonI;		
-		my $CurVal = $this_person_can_choose_n_different_objects{$num_objects}{$person}{'CurVal'};
+	
+	my %choose_person;
+	
+	# first, choose objects that appear a few times
+    foreach my $object ( sort { $count_object{$a} <=> $count_object{$b} } keys %seen_object ){	# || $price_object{$b} <=> $price_object{$a}
+	foreach my $person ( sort { $seen_object{$object}{$a} <=> $seen_object{$object}{$b} } keys %{$seen_object{$object}} ){ # sort { $seen_object{$object}{$a} <=> $seen_object{$object}{$b} }
 		
-		my @objects_with_same_values_full = @{$this_person_can_choose_n_different_objects{$num_objects}{$person}{'objects'}};
-        my @objects_with_same_values_free = grep { not $choose_object{$_} } @objects_with_same_values_full;
-
-		for my $object ( sort { $price_object{$b} <=> $price_object{$a} } @objects_with_same_values_free ){
-			$Opt01ObjForPersonI = $object;
+		next if ( $choose_object{$object} );
+		next if ( $choose_person{$person} );
+	
+		my $CurVal = $seen_object{$object}{$person}; # $CurVal = $current_value{$object} = $Opt01ValForPersonI.
+		my $Opt01ObjForPersonI = $object;
 			
-			$info{$Opt01ObjForPersonI}{'bid'   } = $epsilon;
-			$info{$Opt01ObjForPersonI}{'person'} = $person;
+		$info{$Opt01ObjForPersonI}{'bid'   } = $epsilon;
+		$info{$Opt01ObjForPersonI}{'person'} = $person;
 			
-			$objects_desired_by_this{$person}{$object} = $CurVal;					
-			$choose_object{$object}++;
-			last;
+		$objects_desired_by_this{$person}{$object} = $CurVal;								
+		
+		if ( $verbose >= 8 ){
+		
+			my @objects_with_same_values_full = sort { $a <=> $b } @{$this_person_can_choose_n_different_objects{$person}{'objects'}};
+			my @objects_with_same_values_free = grep { not $choose_object{$_} } @objects_with_same_values_full;
+			
+			printf $output "<*** equal values ***> \$bidForPersonI = \$epsilon = %.5f ; PersonI = %3s chose ObjectJ = %3s between these (@objects_with_same_values_full) --> free (@objects_with_same_values_free) ; \$count_object{$object} = $count_object{$object} \n", $epsilon, $person, $Opt01ObjForPersonI ;		
 		}
 		
-		if ( $verbose >= 8 and defined $Opt01ObjForPersonI ){
-			@objects_with_same_values_full = sort { $a <=> $b } @objects_with_same_values_full;
-			@objects_with_same_values_free = sort { $a <=> $b } @objects_with_same_values_free;
-			printf $output "<*** equal values ***> \$bidForPersonI = \$epsilon = %.5f ; PersonI = %3s chose ObjectJ = %3s between these (@objects_with_same_values_full) --> free (@objects_with_same_values_free) \n", $epsilon, $person, $Opt01ObjForPersonI ;		
-		}
+		$choose_object{$object}++;
+		$choose_person{$person}++;		
 	}}
 	
-	foreach my $object ( keys %info ) # sort { $a <=> $b }
+	foreach my $object ( keys %info ) # sort { $a <=> $b } or sort { $info{$a}{'person'} <=> $info{$b}{'person'} }
 	{
 		my $bid    = $info{$object}{'bid'   };
 		my $person = $info{$object}{'person'};
@@ -1016,7 +1020,7 @@ __END__
 =head1 AUTHOR
 
     Claudio Fernandes de Souza Rodrigues
-	May 2, 2018
+	May 4, 2018
 	Sao Paulo, Brasil
 	claudiofsr@yahoo.com
 	
